package com.ruoyi.web.controller.system;

import java.util.List;
import javax.servlet.http.HttpServletResponse;


import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.system.domain.ProjectManage;
import com.ruoyi.system.domain.ProjectRepayment;
import com.ruoyi.system.mapper.ProjectAnnexMapper;
import com.ruoyi.system.mapper.ProjectRepaymentMapper;
import com.ruoyi.system.service.IProjectManageService;
import com.ruoyi.system.service.IProjectRepaymentService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 项目管理Controller
 */
@RestController
@RequestMapping("/system/project")
public class ProjectManageController extends BaseController
{
    @Autowired
    private IProjectManageService projectManageService;

    @Autowired
    private IProjectRepaymentService projectRepaymentService;

    @Autowired
    private ProjectRepaymentMapper projectRepaymentMapper;

    @Autowired
    private ProjectAnnexMapper projectAnnexMapper;

    /**
     * 查询项目管理列表
     */
    @PreAuthorize("@ss.hasPermi('system:project:list')")
    @GetMapping("/list")
    public TableDataInfo list(ProjectManage projectManage)
    {
        startPage();
        // 查询数据库中所有项目
        List<ProjectManage> list = projectManageService.selectProjectManageList(projectManage);
        for (ProjectManage projectManage1 : list) {
            projectManage1.setRepaymentCount(projectRepaymentMapper.countProjectRepayment(projectManage1.getId()));
            ProjectRepayment projectRepayment = new ProjectRepayment();
            projectRepayment.setProjectId(projectManage1.getId());
            // 设置项目的回款详情
            projectManage1.setRepaymentDetails(projectRepaymentService.selectProjectRepaymentList(projectRepayment));
            projectManage1.setProjectMemberNames(projectManage1.getProjectMemberNames());
            //  设置项目附件
            projectManage1.setProjectAnnexList(projectAnnexMapper.selectProjectAnnexListByProjectId(projectManage1.getId()));
        }
        return getDataTable(list);
    }

    /**
     * 导出项目管理列表
     */
    @PreAuthorize("@ss.hasPermi('system:project:export')")
    @Log(title = "项目管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, ProjectManage projectManage)
    {
        List<ProjectManage> list = projectManageService.selectProjectManageList(projectManage);
        ExcelUtil<ProjectManage> util = new ExcelUtil<ProjectManage>(ProjectManage.class);
        util.exportExcel(response, list, "项目管理数据");
    }

    /**
     * 获取项目管理详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:project:view')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(projectManageService.selectProjectManageById(id));
    }

    /**
     * 新增项目管理
     */
    @PreAuthorize("@ss.hasPermi('system:project:add')")
    @Log(title = "项目管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody ProjectManage projectManage)
    {
        // 项目是否重复
        if (!StringUtils.isEmpty(projectManageService
                .selectProjectManageByProjectName(projectManage.getProjectName())))
        {
            return AjaxResult.error("项目已存在或已存在草稿箱");
        }
        projectManage.setDelFlag(String.valueOf(ProjectManage.NO_DELETE));
        return toAjax(projectManageService.insertProjectManage(projectManage));
    }

    /**
     * 修改项目管理
     */
    @PreAuthorize("@ss.hasPermi('system:project:edit')")
    @Log(title = "项目管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody ProjectManage projectManage)
    {
        return toAjax(projectManageService.updateProjectManage(projectManage));
    }

    /**
     * 删除项目管理
     */
    @PreAuthorize("@ss.hasPermi('system:project:remove')")
    @Log(title = "项目管理", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(projectManageService.deleteProjectManageByIds(ids));
    }

    /*
    * 查询草稿箱项目
    * */
    @PreAuthorize("@ss.hasPermi('system:project:draft')")
    @GetMapping("/draft/list")
    public TableDataInfo draft(ProjectManage projectManage)
    {
        startPage();
        List<ProjectManage> list = projectManageService.selectProjectManageListDraft(projectManage);
        for (ProjectManage projectManage1 : list) {
            // 设置项目的回款详情
            projectManage1.setRepaymentCount(projectRepaymentMapper.countProjectRepayment(projectManage1.getId()));
            ProjectRepayment projectRepayment = new ProjectRepayment();
            projectRepayment.setProjectId(projectManage1.getId());
            projectManage1.setRepaymentDetails(projectRepaymentService.selectProjectRepaymentList(projectRepayment));
            //  设置项目附件
            projectManage1.setProjectAnnexList(projectAnnexMapper.selectProjectAnnexListByProjectId(projectManage1.getId()));
        }
        return getDataTable(list);
    }

    /*
    * 新增草稿箱项目
    * */
    @PreAuthorize("@ss.hasPermi('system:project:add')")
    @Log(title = "项目管理", businessType = BusinessType.INSERT)
    @PostMapping("/draft")
    public AjaxResult addDraft(@RequestBody ProjectManage projectManage)
    {
        // 项目是否重复
        ProjectManage projectManageQuery = new ProjectManage();
        projectManageQuery.setProjectName(projectManage.getProjectName());
        if (!StringUtils.isEmpty(projectManageService.selectProjectManageList(projectManageQuery)))
        {
            return AjaxResult.error("项目已存在或已存在草稿箱");
        }
        // 草稿箱
        projectManage.setDraft(String.valueOf(ProjectManage.DRAFT));
        return toAjax(projectManageService.insertProjectManage(projectManage));
    }

    /*
    * 逻辑删除项目
    * */
    @PreAuthorize("@ss.hasPermi('system:project:logicRemove')")
    @Log(title = "项目管理", businessType = BusinessType.OTHER)
    @DeleteMapping("/logicRemove/{id}")
    public AjaxResult logicRemove(@PathVariable("id") Long id)
    {
        ProjectManage projectManage = new ProjectManage();
        projectManage.setId(id);
        projectManage.setDelFlag(String.valueOf(ProjectManage.DELETE));
        return toAjax(projectManageService.updateProjectManage(projectManage));
    }

    /*
    * 获取项目列表
    * 下拉框
    * */
    @GetMapping("/getProjectList")
    public AjaxResult getProjectList()
    {
        List<ProjectManage> projectManageList = projectManageService.selectProjectManageList(new ProjectManage());
        return AjaxResult.success(projectManageList);
    }

    /*
    * 获取草稿箱项目列表
    * 下拉框
    * */
    @GetMapping("/getDraftProjectList")
    public AjaxResult getDraftProjectList()
    {
        List<ProjectManage> projectManageList = projectManageService.selectProjectManageListDraft(new ProjectManage());
        return AjaxResult.success(projectManageList);
    }
}
